!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                   XEFFORT LIBRARY SAMPLES
!=======================================================================
! OutWindow.f90 - Module for handling output window in XSciCalc sample
! 
! XSciCalc is a simple dialog-based application which launches Notepad
! so that user can edit an input file, does a long calculation, and
! spawns an "output window".
!
! 2005. Xeffort.com
!        Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================
MODULE OutWindow

USE Xeffort
USE DFWIN

IMPLICIT NONE

TYPE(X_WINDOW):: xOutWnd
INTEGER::        idEdit, idStatic

INTEGER::        m_nPoints          !Number of evaluated points
REAL::           m_xPoint(1000),  & !file entries
                 m_yPoint(0:1000)   !Calculation results (used to draw graph)
REAL::           m_yMin, m_yMax     !Min & max values (used to scale the graph)

INTEGER, PARAMETER:: WM_OUTWINDOWCLOSED = WM_APP+1

!=======================================================================
CONTAINS
!=======================================================================
!Create the output window and its child windows (read-only edit control and 
!static owner-drawn control)
SUBROUTINE CreateOutputWindow()

LOGICAL:: bSt
INTEGER:: iret, iX, iY, iWidth, iHeight

bSt = XGetWindowRect(XW_FRAME, iX, iY, iWidth, iHeight)

bSt = XCreateWindow(xOutWnd, XW_FRAME, WS_POPUP.OR.WS_OVERLAPPEDWINDOW.OR.WS_VISIBLE, &
       "Output Window", 0, iX+iWidth, iY+iHeight/2 - 300, iWidth, 600)

!Create a multi-line edit control
bSt = XGetClientRect(xOutWnd, iWidth, iHeight)
idEdit = XCtlCreate(xOutWnd, 1, XCTL_EDIT, CTS_MULTILINE.OR.CTS_READONLY, 0, 0, iWidth, iHeight/2)
bSt = XCtlSet(xOutWnd, idEdit, .TRUE., CTL_VISIBLE)

!Create a static owner-drawn control to draw onto. 
!Instead, we could use a WM_PAINT handler but this method is slightly
!more suitable because it correctly clips & confines the graphic output.
idStatic = XCtlCreate(xOutWnd, 2, XCTL_STATIC, CTS_OWNERDRAW, 0, iHeight/2, iWidth, iHeight/2)
bSt = XCtlSetSub(xOutWnd, idStatic, xOutWnd_OnDrawStatic, CTL_DRAWITEM)

!Take care to resize the controls when the xOutWnd is resized:
bSt = XSetHandler(xOutWnd, WM_SIZE, xOutWnd_OnSize)

!Since we don't know when exactly the window will be closed,
!we must write a handler to enable the XW_FRAME back:
bSt = XSetHandler(xOutWnd, WM_DESTROY, xOutWnd_OnDestroy)

END SUBROUTINE CreateOutputWindow
!=======================================================================
!Appends text to the edit control.
SUBROUTINE xOutWnd_WriteText(sText)

CHARACTER(*):: sText
LOGICAL:: bSt

bSt = XCtlSet(xOutWnd, idEdit, sText, 0)

END SUBROUTINE xOutWnd_WriteText
!=======================================================================
!Sizing callback. It re-tiles the idEdit and idStatic windows when
!the parent is 
LOGICAL FUNCTION xOutWnd_OnSize(xWnd, iWidth, iHeight, nFlag)
 
TYPE(X_WINDOW)::              xWnd
INTEGER, INTENT(IN)::         iWidth, iHeight   
INTEGER, INTENT(IN)::         nFlag             

INTEGER::                     iret

iret = XCtlMove(xWnd, idEdit, 0, 0, iWidth, iHeight/2)
iret = XCtlMove(xWnd, idStatic, 0, iHeight/2 + 1, iWidth, iHeight/2)

xOutWnd_OnSize = .FALSE.

END FUNCTION xOutWnd_OnSize
!=======================================================================
!Callback called when xOutWnd is about to be destroyed.
LOGICAL FUNCTION xOutWnd_OnDestroy(xWnd)
 
TYPE(X_WINDOW)::           xWnd

INTEGER::                  iret

!Post a app-defined message to exit any message loops which may
!be waiting for xOutWnd to be closed.
iret = XPostMessage(XW_FRAME, WM_OUTWINDOWCLOSED, 0, 0)
!Continue destruction normally
xOutWnd_OnDestroy = .FALSE.   

END FUNCTION xOutWnd_OnDestroy
!=======================================================================
!Draws the graph. It is invoked whenever the idStatic or its parent
!is to be updated.
SUBROUTINE xOutWnd_OnDrawStatic(xWnd, idCmd, hDC)
 
TYPE(X_WINDOW)::           xWnd
INTEGER, INTENT(IN)::      idCmd 
INTEGER, INTENT(IN)::      hDC

LOGICAL::                  bSt
INTEGER::                  iX1, iY1, iX2, iY2, i
TYPE(X_DC)::               xDC; POINTER(pDC, xDC)

pDC = LOC(hDC)
bSt = XFillViewport(xDC, X_BRUSH(BS_SOLID, XCOLOR_WHITE, 0))

!Set the 1st window to be proportional to the entire area
CALL XSetWindow(xDC, 0., 0., 100., 100.)
bSt = XLine_W(xDC, 5., 5., 5., 95.)
bSt = XLine_W(xDC, 5., 95., 95., 95.)

!Set the 2nd window to the diagram size
CALL XWnd2View(xDC, 5., 5., iX1, iY1)
CALL XWnd2View(xDC, 95., 95., iX2, iY2)
CALL XSetViewport(xDC, iX1, iY1, iX2, iY2)
CALL XSetWindow(xDC, 0., m_yMax, 100., 0.)

!Draw the diagram
CALL XSetPen(xDC, XCOLOR_RED)
CALL XMoveTo_W(xDC, 0., m_YPoint(1))
DO i=2,m_nPoints
   bSt = XLineTo_W(xDC, REAL(i), m_YPoint(i))
END DO

END SUBROUTINE xOutWnd_OnDrawStatic

END MODULE OutWindow